@@ -5,6 +5,9 @@ import yaml
 import pandas as pd
 import tabulate
 import logging
+import email
+import smtplib
+import ssl
 from pyats.topology import Testbed, Device
 from genie import testbed
 from rich import print_json
@@ -13,21 +16,33 @@ from json2table import convert
 from pathlib import Path
 from jinja2 import Environment, FileSystemLoader
 from fpdf import FPDF
+from email import encoders
+from email.mime.base import MIMEBase
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
 
 class GetJson():
-    def __init__(self, hostname, os, username, password, command, filetype):
+    def __init__(self, hostname, os, username, password, command, filetype, from_email, email_password, to_email):
         self.hostname = hostname
         self.os = os
         self.username = username
         self.password = password
         self.command = command
         self.filetype = filetype
+        self.from_email = from_email
+        self.email_password = email_password
+        self.to_email = to_email
 
     def print_json(self):
         parsed_json = json.dumps(self.capture_state(), indent=4, sort_keys=True)
         print_json(parsed_json)
-        if self.filetype:
-            self.pick_filetype(parsed_json)
+        if "Cannot" in parsed_json:
+            click.secho("No Data To Create File", fg='red')
+        else:
+            if self.filetype:
+                self.pick_filetype(parsed_json)
+            if self.from_email != 'none' and self.email_password != 'none' and self.to_email != 'none':
+                self.send_email(parsed_json)            
 
     def pick_filetype(self, parsed_json):
         if self.filetype == "none":
@@ -76,7 +91,7 @@ class GetJson():
         env = Environment(loader=FileSystemLoader(template_dir))
         datatable_template = env.get_template('datatable.j2')
         datatable_output = datatable_template.render(table=html_version)
-        with open(f'{self.hostname} {self.command}_datatable.html', 'w') as f:
+        with open(f'{self.hostname} {self.command}.html', 'w') as f:
             f.write(datatable_output)
         click.secho(f"Datatable file created at { sys.path[0] }/{self.hostname} {self.command}.html", fg='green')
 
@@ -96,11 +111,60 @@ class GetJson():
             f.write(markdown_table)
         click.secho(f"Markdown file created at { sys.path[0] }/{self.hostname} {self.command}.md", fg='green')
 
-    def csv_file(self, parsed_json, expand_all=False):
-        df = pd.json_normalize(json.loads(parsed_json))
-        df.to_csv(f'{self.hostname} {self.command}.csv', index=False)
+    def csv_file(self, parsed_json):
+        supported_templates = ['acl','arp','bgp','dot1x','hsrp','interface','lldp','ntp','ospf','platform','routing','stp','vlan','vrf','show cdp neighbors','show ip interface brief']
+        for template in supported_templates:
+            if self.command == template:
+                template_dir = Path(__file__).resolve().parent
+                env = Environment(loader=FileSystemLoader(template_dir))
+                csv_template = env.get_template(f'{self.os} csv.j2')              
+                csv_output = csv_template.render(command = self.command, data_to_template=json.loads(parsed_json))
+                with open(f'{self.hostname} {self.command}.csv', 'w') as f:
+                    f.write(csv_output)
+                break          
+            else:
+                df = pd.json_normalize(json.loads(parsed_json))
+                df.to_csv(f'{self.hostname} {self.command}.csv', index=False)
         click.secho(f"CSV file created at { sys.path[0] }/{self.hostname} {self.command}.csv", fg='green')
 
+    def send_email(self, parsed_json):
+        subject = f"An Email from pyYAHTS Device {self.hostname} and Command {self.command}"
+        body = parsed_json
+        sender_email = self.from_email
+        receiver_email = self.to_email
+        password = self.email_password
+
+        message = MIMEMultipart()
+        message["From"] = sender_email
+        message["To"] = receiver_email
+        message["Subject"] = subject
+
+        message.attach(MIMEText(body, "plain"))
+
+        if self.filetype:
+            print(self.filetype)
+            filename = f'{self.hostname} {self.command}.{self.filetype}'
+            with open(filename, "rb") as attachment:
+                part = MIMEBase("application", "octet-stream")
+                part.set_payload(attachment.read())
+                
+            encoders.encode_base64(part)
+            part.add_header(
+                "Content-Disposition",
+                f"attachment; filename= {filename}",
+            )
+            message.attach(part)
+            text = message.as_string()
+        else:
+            print("if filetype is none")
+            text = message.as_string()
+
+        context = ssl.create_default_context()
+        with smtplib.SMTP_SSL("smtp.gmail.com", 465, context=context) as server:
+            server.login(sender_email, password)
+            server.sendmail(sender_email, receiver_email, text)
+            click.secho(f"Email sent to {receiver_email}", fg='green')
+
     # Create Testbed
     def connect_device(self):
         try:
@@ -147,17 +211,17 @@ class GetJson():
                 try:
                     command_output = device.learn(self.command)
                 except:
-                    command_output = "Cannot Learn Config"
+                    command_output = f"Cannot Parse { self.command }"
             elif self.command == "platform":
                 try:
                     command_output = device.learn(self.command).to_dict()
                 except:
-                    command_output = "Cannot Learn Platform"
+                    command_output = f"Cannot Parse { self.command }"
             else:
                 try:
                     command_output = device.learn(self.command).info
                 except:
-                    command_output = f"Cannot Learn { self.command }"
+                    command_output = f"Cannot Parse { self.command }"
             return(command_output)
 
 @click.command()
@@ -166,9 +230,12 @@ class GetJson():
 @click.option('--username', prompt='Username', help='Username', required=True)
 @click.option('--password', prompt=True, hide_input=True, help="User Password", required=True)
 @click.option('--command', prompt='Command', help='A valid pyATS Learn Function (i.e. ospf) or valid CLI Show Command (i.e. "show ip interface brief")', required=True)
-@click.option('--filetype', prompt='Filetype', type=click.Choice(['none','json','yaml','html','datatable','markdown','pdf','csv'], case_sensitive=True), help='Filetype to output captured network state to', required=False, default='none')
-def cli(hostname, os, username, password, command, filetype):
-    invoke_class = GetJson(hostname, os, username, password, command, filetype)
+@click.option('--filetype', prompt='Filetype', type=click.Choice(['none','json','yaml','html','markdown','pdf','csv'], case_sensitive=True), help='Filetype to output captured network state to', required=False, default='none')
+@click.option('--from_email', prompt='From Email', help='Email address to send output from', required=False, default='none')
+@click.option('--email_password', prompt='Email Password', hide_input=True, help='Email account password', required=False, default='none')
+@click.option('--to_email', prompt='To Email', help='Email address to send output to', required=False, default='none')
+def cli(hostname, os, username, password, command, filetype, from_email, email_password, to_email):
+    invoke_class = GetJson(hostname, os, username, password, command, filetype, from_email, email_password, to_email)
     invoke_class.print_json()
 
 if __name__ == "__main__":
